/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *当前活动便签项
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.model;

//在model包中
import android.appwidget.AppWidgetManager;//以下下为引用的一些库文件
import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;//游标的使用
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.data.Notes;//小米便签操作的整体属性
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;
import net.micode.notes.tool.ResourceParser.NoteBgResources;

public class WorkingNote {
    // 申明WorkingNote类,创建小米便签的主要类，包括创建空便签，保存便签，加载小米便签内容，和设置小米便签的一些小部件之类的操作
    // Note for the working note
    // 下列定义了一系列变量用于设置便签的不同属性以及相关功能
    private Note mNote;
    // Note Id
    private long mNoteId;
    // Note content
    private String mContent;
    // Note mode
    private int mMode;
    // 是否是清单模式
    private long mAlertDate;
    // 设置闹钟时间
    private long mModifiedDate;
    // 最后修改时间
    private int mBgColorId;
    // 背景颜色ID
    private int mWidgetId;
    // 控件ID
    private int mWidgetType;
    // 控件类型，有Notes.TYPE_WIDGET_INVALIDE(-1)、TYPE_WIDGET_2X(0)、TYPE_WIDGET_4X(1)三种
    private long mFolderId;
    //// 便签文件夹ID
    private Context mContext;
    // 当前便签的上下文
    private static final String TAG = "WorkingNote";
    // .声明 DATA_PROJECTION字符串数组
    private boolean mIsDeleted;
    // 是否应该被删除
    private NoteSettingChangedListener mNoteSettingStatusListener;
    // 一个用来监听设置是否有变化的接口
    // 声明 DATA_PROJECTION字符串数组
    public static final String[] DATA_PROJECTION = new String[] {

            // 声明 NOTE_PROJECTION字符串数组
            DataColumns.ID,
            DataColumns.CONTENT,
            DataColumns.MIME_TYPE,
            DataColumns.DATA1,
            DataColumns.DATA2,
            DataColumns.DATA3,
            DataColumns.DATA4,
    };// 声明各个常量

    // 声明 NOTE_PROJECTION字符串数组
    public static final String[] NOTE_PROJECTION = new String[] {
            // 保存便签自身属性的字符串数组
            NoteColumns.PARENT_ID,
            NoteColumns.ALERTED_DATE,
            NoteColumns.BG_COLOR_ID,
            NoteColumns.WIDGET_ID,
            NoteColumns.WIDGET_TYPE,
            NoteColumns.MODIFIED_DATE
    };

    private static final int DATA_ID_COLUMN = 0;
    // 规定每一个数据类型在哪一行
    private static final int DATA_CONTENT_COLUMN = 1;
    private static final int DATA_MIME_TYPE_COLUMN = 2;
    private static final int DATA_MODE_COLUMN = 3;
    private static final int NOTE_PARENT_ID_COLUMN = 0;
    // 以下6个常量表示便签投影的0-5列
    private static final int NOTE_ALERTED_DATE_COLUMN = 1;
    private static final int NOTE_BG_COLOR_ID_COLUMN = 2;
    private static final int NOTE_WIDGET_ID_COLUMN = 3;
    private static final int NOTE_WIDGET_TYPE_COLUMN = 4;
    private static final int NOTE_MODIFIED_DATE_COLUMN = 5;

    /*
     * 作用：根据上下文和当前文件夹的ID，初始化当前编辑中的便签
     * 实现：设置便签有关属性的信息的初始值
     * 参数：@context：当前活动的上下文
     * 
     * @folderId：当前文件夹的ID
     */
    // New note construct
    public WorkingNote(Context context, long folderId) {
        mContext = context;
        mAlertDate = 0;
        mModifiedDate = System.currentTimeMillis();
        // 系统现在的时间,时间的表达格式为当前计算机时间和GMT时间(格林威治时间)1970年1月1号0时0分0秒所差的毫秒数
        mFolderId = folderId;
        // 默认最后一次修改日期为当前时间
        mNote = new Note();
        // 加载一个已存在的便签
        mNoteId = 0;
        mIsDeleted = false;
        // 没有提供id所以默认为0
        mMode = 0;
        mWidgetType = Notes.TYPE_WIDGET_INVALIDE;
        // 默认是不可见
    }

    // WorkingNote的构造函数
    // Existing note construct
    private WorkingNote(Context context, long noteId, long folderId) {
        // 加载Note
        // 该方法初始化类里的各项变量
        mContext = context;
        // 调用query函数找到第一个条目
        mNoteId = noteId;
        mFolderId = folderId;
        mIsDeleted = false;
        mNote = new Note();
        loadNote();
        // 加载便签
    }

    // 加载Note
    // 通过数据库调用query函数找到第一个条目
    private void loadNote() {
        // 加载已有的便签
        Cursor cursor = mContext.getContentResolver().query(
                // 存在第一个条目
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, mNoteId), NOTE_PROJECTION, null,
                null, null);

        // 若存在，储存相应信息
        if (cursor != null) {
            // 通过数据库调用query函数找到第一个条目
            if (cursor.moveToFirst()) {
                // moveToFirst：will return false if the cursor is empty
                mFolderId = cursor.getLong(NOTE_PARENT_ID_COLUMN);
                mBgColorId = cursor.getInt(NOTE_BG_COLOR_ID_COLUMN);// 关闭cursor游标
                mWidgetId = cursor.getInt(NOTE_WIDGET_ID_COLUMN);
                mWidgetType = cursor.getInt(NOTE_WIDGET_TYPE_COLUMN);
                mAlertDate = cursor.getLong(NOTE_ALERTED_DATE_COLUMN);
                mModifiedDate = cursor.getLong(NOTE_MODIFIED_DATE_COLUMN);
            }
            cursor.close();
            // 若不存在，报错
        } else {
            Log.e(TAG, "No note with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note with id " + mNoteId);
        }
        loadNoteData();
    }

    // 加载NoteData
    private void loadNoteData() {
        Cursor cursor = mContext.getContentResolver().query(Notes.CONTENT_DATA_URI, DATA_PROJECTION,
                DataColumns.NOTE_ID + "=?", new String[] {
                        String.valueOf(mNoteId)
                }, null);

        if (cursor != null) {
            // 查到信息不为空
            if (cursor.moveToFirst()) { // 查看第一项是否存在
                do {
                    String type = cursor.getString(DATA_MIME_TYPE_COLUMN);
                    if (DataConstants.NOTE.equals(type)) {
                        mContent = cursor.getString(DATA_CONTENT_COLUMN);
                        mMode = cursor.getInt(DATA_MODE_COLUMN);
                        mNote.setTextDataId(cursor.getLong(DATA_ID_COLUMN));
                    } else if (DataConstants.CALL_NOTE.equals(type)) {
                        mNote.setCallDataId(cursor.getLong(DATA_ID_COLUMN));
                    } else {
                        Log.d(TAG, "Wrong note type with type:" + type);
                    }
                } while (cursor.moveToNext());// 查阅所有项,直到为空
            }
            cursor.close();
        } else {
            Log.e(TAG, "No data with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note's data with id " + mNoteId);
        }
    }

    // 创建空的Note
    // 传参：context，文件夹id，widget，背景颜色
    public static WorkingNote createEmptyNote(Context context, long folderId, int widgetId,
            int widgetType, int defaultBgColorId) {
        WorkingNote note = new WorkingNote(context, folderId);
        // 设定相关属性
        note.setBgColorId(defaultBgColorId);
        note.setWidgetId(widgetId);
        note.setWidgetType(widgetType);
        return note;
    }

    public static WorkingNote load(Context context, long id) {
        return new WorkingNote(context, id, 0);
    }

    // 保存Note
    public synchronized boolean saveNote() {
        if (isWorthSaving()) { // 是否值得保存
            if (!existInDatabase()) { // 是否存在数据库中
                if ((mNoteId = Note.getNewNoteId(mContext, mFolderId)) == 0) {
                    Log.e(TAG, "Create new note fail with id:" + mNoteId);
                    return false;
                }
            }

            mNote.syncNote(mContext, mNoteId);

            /**
             * Update widget content if there exist any widget of this note
             */
            if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                    && mWidgetType != Notes.TYPE_WIDGET_INVALIDE
                    && mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onWidgetChanged();
            }
            return true;
        } else {
            return false;
        }
    }

    // 是否在数据库中存在
    public boolean existInDatabase() {
        return mNoteId > 0;
    }

    // 是否值得保存
    private boolean isWorthSaving() {
        // 被删除，或（不在数据库中 内容为空），或 本地已保存过
        if (mIsDeleted || (!existInDatabase() && TextUtils.isEmpty(mContent))
                || (existInDatabase() && !mNote.isLocalModified())) {
            return false;
        } else {
            return true;
        }
    }

    // 设置mNoteSettingStatusListener
    public void setOnSettingStatusChangedListener(NoteSettingChangedListener l) {
        mNoteSettingStatusListener = l;
    }

    // 设置AlertDate
    // 若 mAlertDate与data不同，则更改mAlertDate并设定NoteValue
    public void setAlertDate(long date, boolean set) {
        if (date != mAlertDate) {
            mAlertDate = date;
            mNote.setNoteValue(NoteColumns.ALERTED_DATE, String.valueOf(mAlertDate));
        }
        if (mNoteSettingStatusListener != null) {
            mNoteSettingStatusListener.onClockAlertChanged(date, set);
        }
    }

    // 设定删除标记
    public void markDeleted(boolean mark) {
        // 设定标记
        mIsDeleted = mark;
        if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                && mWidgetType != Notes.TYPE_WIDGET_INVALIDE && mNoteSettingStatusListener != null) {
            mNoteSettingStatusListener.onWidgetChanged();
            // 调用mNoteSettingStatusListener的 onWidgetChanged方法
        }
    }

    // 设定背景颜色
    public void setBgColorId(int id) {
        if (id != mBgColorId) { // 设定条件 id != mBgColorId
            mBgColorId = id;
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onBackgroundColorChanged();
            }
            mNote.setNoteValue(NoteColumns.BG_COLOR_ID, String.valueOf(id));
        }
    }

    // 设定检查列表模式
    // 参数：mode
    public void setCheckListMode(int mode) {
        if (mMode != mode) { // 设定条件 mMode != mode
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onCheckListModeChanged(mMode, mode);
            }
            mMode = mode;
            mNote.setTextData(TextNote.MODE, String.valueOf(mMode));
        }
    }

    // 设定WidgetType
    // 参数：type
    public void setWidgetType(int type) {
        if (type != mWidgetType) {// 设定条件 type != mWidgetType
            mWidgetType = type;
            mNote.setNoteValue(NoteColumns.WIDGET_TYPE, String.valueOf(mWidgetType));
            // 调用Note的setNoteValue方法更改WidgetType
        }
    }

    // 设定WidgetId
    // 参数：id
    public void setWidgetId(int id) {
        if (id != mWidgetId) {// 设定条件 id != mWidgetId
            mWidgetId = id;
            mNote.setNoteValue(NoteColumns.WIDGET_ID, String.valueOf(mWidgetId));
            // 调用Note的setNoteValue方法更改WidgetId
        }
    }

    // 设定WorkingTex
    // 参数：更改的text
    public void setWorkingText(String text) {
        if (!TextUtils.equals(mContent, text)) {// 设定条件 mContent, text内容不同
            mContent = text;
            mNote.setTextData(DataColumns.CONTENT, mContent);
            // 调用Note的setTextData方法更改WorkingText
        }
    }

    // 转变mNote的CallData及CallNote信息
    // 参数：String phoneNumber, long callDate
    public void convertToCallNote(String phoneNumber, long callDate) {
        mNote.setCallData(CallNote.CALL_DATE, String.valueOf(callDate));
        mNote.setCallData(CallNote.PHONE_NUMBER, phoneNumber);
        mNote.setNoteValue(NoteColumns.PARENT_ID, String.valueOf(Notes.ID_CALL_RECORD_FOLDER));
    }

    // 判断是否有时钟题型
    public boolean hasClockAlert() {
        return (mAlertDate > 0 ? true : false);
    }

    // 获取Content
    public String getContent() {
        return mContent;
    }

    // 获取AlertDate
    public long getAlertDate() {
        return mAlertDate;
    }

    // 获取ModifiedDate
    public long getModifiedDate() {
        return mModifiedDate;
    }

    // 获取背景颜色来源id
    public int getBgColorResId() {
        return NoteBgResources.getNoteBgResource(mBgColorId);
    }

    // 获取背景颜色id
    public int getBgColorId() {
        return mBgColorId;
    }

    // 获取标题背景颜色id
    public int getTitleBgResId() {
        return NoteBgResources.getNoteTitleBgResource(mBgColorId);
    }

    // 获取CheckListMode
    public int getCheckListMode() {
        return mMode;
    }

    // 获取便签id
    public long getNoteId() {
        return mNoteId;
    }

    // 获取文件夹id
    public long getFolderId() {
        return mFolderId;
    }

    // 获取WidgetId
    public int getWidgetId() {
        return mWidgetId;
    }

    // 获取WidgetType
    public int getWidgetType() {
        return mWidgetType;
    }

    // 创建接口 NoteSettingChangedListener,便签更新监视
    // 为NoteEditActivity提供接口
    // 提供函数有
    public interface NoteSettingChangedListener {
        /**
         * Called when the background color of current note has just changed
         */
        void onBackgroundColorChanged();

        /**
         * Called when user set clock
         */
        void onClockAlertChanged(long date, boolean set);

        /**
         * Call when user create note from widget
         */
        void onWidgetChanged();

        /**
         * Call when switch between check list mode and normal mode
         * 
         * @param oldMode is previous mode before change
         * @param newMode is new mode
         */
        void onCheckListModeChanged(int oldMode, int newMode);
    }
}
