//表示在哪个包，依赖关系
package net.micode.notes.data;
//uri是统一资源标识符，涉及到安卓权限

import android.net.Uri;

//Notes 类中定义了很多常量，这些常量大多是int型和string型
public class Notes {
    // 定义了一个权限名，这里是为了符合Uri的格式而定义的
    public static final
    String AUTHORITY = "micode_notes";
    //  设置标签，表示APP的名称是Notes
    public static final String TAG = "Notes";

    //以下三个常量对NoteColumns.TYPE的值进行设置时会用到
    public static final int TYPE_NOTE = 0;
    public static final int TYPE_FOLDER = 1;
    public static final int TYPE_SYSTEM = 2;

    /**
     * Following IDs are system folders' identifiers
     * {@link Notes#ID_ROOT_FOLDER } is default folder
     * {@link Notes#ID_TEMPARAY_FOLDER } is for notes belonging no folder
     * {@link Notes#ID_CALL_RECORD_FOLDER} is to store call records
     */
//垃圾文件夹
    public static final int ID_ROOT_FOLDER = 0;
    //临时警报
    public static final int ID_TEMPARAY_FOLDER = -1;
    //通话记录文件夹
    public static final int ID_CALL_RECORD_FOLDER = -2;
    //垃圾文件夹3
    public static final int ID_TRASH_FOLER = -3;
//定义string字符串
    public static final String INTENT_EXTRA_ALERT_DATE =

            "net.micode.notes.alert_date";
    public static final String INTENT_EXTRA_BACKGROUND_ID =

            "net.micode.notes.background_color_id";
    public static final String INTENT_EXTRA_WIDGET_ID =

            "net.micode.notes.widget_id";
    public static final String INTENT_EXTRA_WIDGET_TYPE =

            "net.micode.notes.widget_type";
    public static final String INTENT_EXTRA_FOLDER_ID =

            "net.micode.notes.folder_id";
    public static final String INTENT_EXTRA_CALL_DATE =

            "net.micode.notes.call_date";
//定义查询便签和指针
    public static final int TYPE_WIDGET_INVALIDE = -1;
    public static final int TYPE_WIDGET_2X = 0;
    public static final int TYPE_WIDGET_4X = 1;
//DataContants类：存放textnotes和callnotes地址
    public static class DataConstants {
        public static final String NOTE = TextNote.CONTENT_ITEM_TYPE;
        public static final String CALL_NOTE = CallNote.CONTENT_ITEM_TYPE;
    }

    /**
     * Uri to query all notes and folders
     */
//创建URI访问实例parse中的参数分三部分，scheme（访问资源的命名机制），authority（存放资源的主机名），path（资源自身的名称）
    public static final Uri CONTENT_NOTE_URI = Uri.parse("content://" +

            AUTHORITY + "/note");//定义查询便签和文件夹的指针。

//    public static final Uri my_URI = ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI , 10);

    /**
     * Uri to query data
     * 系统根据data查询所需的Uri常量
     */
    public static final Uri CONTENT_DATA_URI = Uri.parse("content://" +

            AUTHORITY + "/data");//定义查找数据的指针。

    // 定义NoteColumns的常量,用于后面创建数据库的表头
    public interface NoteColumns {
        /**
         * The unique ID for a row
         * <P> Type: INTEGER (long) </P>
         */
        public static final String ID = "_id";

        /**
         * The parent's id for note or folder
         * <P> Type: INTEGER (long) </P>
         */
        //父节点的ID
        public static final String PARENT_ID = "parent_id";

        /**
         * Created data for note or folder
         * <P> Type: INTEGER (long) </P>
         * 创建时间
         */
        public static final String CREATED_DATE = "created_date";

        /**
         * Latest modified date
         * <P> Type: INTEGER (long) </P>
         * 提醒时间
         */
        public static final String MODIFIED_DATE = "modified_date";


        /**
         * Alert date
         * <P> Type: INTEGER (long) </P>
         * 提醒时间
         */
        public static final String ALERTED_DATE = "alert_date";

        /**
         * Folder's name or text content of note
         * <P> Type: TEXT </P>
         * 文件夹名字或文件内容
         */
        public static final String SNIPPET = "snippet";

        /**
         * Note's widget id
         * <P> Type: INTEGER (long) </P>
         * 小部件ID
         */
        public static final String WIDGET_ID = "widget_id";

        /**
         * Note's widget type
         * <P> Type: INTEGER (long) </P>
         * 小部件类型
         */
        public static final String WIDGET_TYPE = "widget_type";

        /**
         * Note's background color's id
         * <P> Type: INTEGER (long) </P>
         * 背景颜色ID
         */
        public static final String BG_COLOR_ID = "bg_color_id";

        /**
         * For text note, it doesn't has attachment, for multi-media
         * note, it has at least one attachment
         * <P> Type: INTEGER </P>
         * 是否有附件
         */
        public static final String HAS_ATTACHMENT = "has_attachment";

        /**
         * Folder's count of notes
         * <P> Type: INTEGER (long) </P>
         * 文件夹中标签的数量
         */
        public static final String NOTES_COUNT = "notes_count";

        /**
         * The file type: folder or note
         * <P> Type: INTEGER </P>
         * 文件或者便签的类型
         */
        public static final String TYPE = "type";

        /**
         * The last sync id
         * <P> Type: INTEGER (long) </P>
         * 最后一次同步的ID
         */
        public static final String SYNC_ID = "sync_id";//同步

        /**
         * Sign to indicate local modified or not
         * <P> Type: INTEGER </P>
         * 标识本地是否修改
         */
        public static final String LOCAL_MODIFIED = "local_modified";

        /**
         * Original parent id before moving into temporary folder
         * <P> Type : INTEGER </P>
         * 移动到当前文件夹之前的父文件夹ID
         */
        public static final String ORIGIN_PARENT_ID = "origin_parent_id";

        /**
         * The gtask id
         * <P> Type : TEXT </P>
         * 后台任务ID
         */
        public static final String GTASK_ID = "gtask_id";

        /**
         * The version code
         * <P> Type : INTEGER (long) </P>
         * 版本代号
         */
        public static final String VERSION = "version";
    }//这些常量主要是定义便签的属性的。

    // 定义DataColumns的常量,用于后面创建数据库的表头
    public interface DataColumns {
        /**
         * The unique ID for a row
         * <P> Type: INTEGER (long) </P>
         * 一行独立的便签ID
         */
        public static final String ID = "_id";

        /**
         * The MIME type of the item represented by this row.
         * <P> Type: Text </P>
         * MIME类型能包含视频、图像、文本、音频、应用程序等数据
         */
        public static final String MIME_TYPE = "mime_type";

        /**
         * The reference id to note that this data belongs to
         * <P> Type: INTEGER (long) </P>
         * 便签ID
         */
        public static final String NOTE_ID = "note_id";

        /**
         * Created data for note or folder
         * <P> Type: INTEGER (long) </P>
         * 创建时间
         */
        public static final String CREATED_DATE = "created_date";

        /**
         * Latest modified date
         * <P> Type: INTEGER (long) </P>
         * 最后的修改日期
         */
        public static final String MODIFIED_DATE = "modified_date";

        /**
         * Data's content
         * <P> Type: TEXT </P>
         * 数据的内容
         */
        public static final String CONTENT = "content";


        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific,
         * <p>
         * used for
         * integer data type
         * <P> Type: INTEGER </P>
         */
        public static final String DATA1 = "data1";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific,
         * <p>
         * used for
         * integer data type
         * <P> Type: INTEGER </P>
         */
        public static final String DATA2 = "data2";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific,
         * <p>
         * used for
         * TEXT data type
         * <P> Type: TEXT </P>
         */
        public static final String DATA3 = "data3";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific,
         * <p>
         * used for
         * TEXT data type
         * <P> Type: TEXT </P>
         */
        public static final String DATA4 = "data4";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific,
         * <p>
         * used for
         * TEXT data type
         * <P> Type: TEXT </P>
         */
        public static final String DATA5 = "data5";
    }//主要是定义存储便签内容数据的

//文本数据（TextNote）通过关键字implements来实现接口，继承了DataColumns的所有定义常量
    public static final class TextNote implements DataColumns {
        /**
         * Mode to indicate the text in check list mode or not
         * <P> Type: Integer 1:check list mode 0: normal mode </P>
         */
        public static final String MODE = DATA1;

        public static final int MODE_CHECK_LIST = 1;

        public static final String CONTENT_TYPE =

                "vnd.android.cursor.dir/text_note";

        public static final String CONTENT_ITEM_TYPE =

                "vnd.android.cursor.item/text_note";
// 访问该content_provider的URI，parse方法返回的是一个URI类型。通过这个uri可以访问一个网络上或者本地上的资源
        public static final Uri CONTENT_URI = Uri.parse("content://" +

                AUTHORITY + "/text_note");
    }
    //文本内容的数据结构
//TextNote作为DataColumns的子类，进行扩展，用于记录通话数据的表头
    public static final class CallNote implements DataColumns {
        /**
         * Call date for this record
         * <P> Type: INTEGER (long) </P>
         */
        public static final String CALL_DATE = DATA1;

        /**
         * Phone number for this record
         * <P> Type: TEXT </P>
         */
        public static final String PHONE_NUMBER = DATA3;

        public static final String CONTENT_TYPE =

                "vnd.android.cursor.dir/call_note";

        public static final String CONTENT_ITEM_TYPE =

                "vnd.android.cursor.item/call_note";

        public static final Uri CONTENT_URI = Uri.parse("content://" +

                AUTHORITY + "/call_note");
    }//电话内容的数据结构
}